# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""Celery Tasks."""

from __future__ import absolute_import, print_function

import elasticsearch
from celery import shared_task
from celery.utils.log import get_task_logger
from invenio_search import current_search_client
from invenio_workflows.api import WorkflowObject
from invenio_workflows.errors import WorkflowsError

from .proxies import workflow_api_class


LOGGER = get_task_logger(__name__)


@shared_task(ignore_result=True)
def resolve_actions(object_ids, action, *args, **kwargs):
    """Resolve a set of actions."""
    from invenio_workflows_ui import workflow_api_class

    for id_object in object_ids:
        workflow_ui_object = workflow_api_class.get_record(id_object)
        if workflow_ui_object:
            getattr(workflow_ui_object, action)(*args, **kwargs)


@shared_task(ignore_result=False)
def batch_reindex(workflow_ids, request_timeout):
    """Task for bulk reindexing workflow records."""
    indexer = workflow_api_class.indexer

    def actions():
        for workflow_id in workflow_ids:
            try:
                workflow_object = WorkflowObject.get(workflow_id)
                record = workflow_api_class.record_from_object(workflow_object)
                workflow_api_object = workflow_api_class(
                    record,
                    workflow=workflow_object,
                )
                index, doc_type = indexer.record_to_index(workflow_api_object)
                body = indexer._prepare_record(
                    workflow_api_object,
                    index,
                    doc_type,
                )
                yield {
                    '_id': workflow_api_object.id,
                    '_index': index,
                    '_type': doc_type,
                    '_op_type': 'index',
                    '_source': body,
                }
            except WorkflowsError as e:
                LOGGER.warn('Workflow %s failed to load: %s', workflow_id, e)

    try:
        success, failures = elasticsearch.helpers.bulk(
            current_search_client,
            actions(),
            request_timeout=request_timeout,
            raise_on_error=False,
            raise_on_exception=False,
        )
    except Exception as e:
        return {
            'failures': [{
                'error': repr(e),
                'workflow_ids': workflow_ids,
            }],
        }

    return {
        'success': success,
        'failures': failures,
    }



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Deposit actions."""

from __future__ import absolute_import, print_function

import os
import json

from functools import partial, wraps

from flask import (
    abort,
    Blueprint,
    current_app,
    make_response,
    request,
    url_for,
)

from invenio_db import db
from invenio_rest import ContentNegotiatedMethodView
from invenio_rest.errors import RESTException
from invenio_search import RecordsSearch
from flask_login import current_user

from invenio_workflows.errors import WorkflowsMissingObject

from ..search import default_search_factory
from ..tasks import resolve_actions
from ..utils import obj_or_import_string
from ..proxies import workflow_api_class
from ..permissions import action_read_permission, action_write_permission


def create_blueprint(config, context_processors):
    """Create Invenio-Deposit-REST blueprint with all views."""
    blueprint = Blueprint(
        'invenio_workflows_rest',
        __name__,
        url_prefix='',
    )

    workflow_object_serializers = config.get('workflow_object_serializers')
    search_serializers = config.get('search_serializers')
    action_serializers = config.get('action_serializers')
    file_serializers = config.get('file_serializers')
    bulk_action_serializers = config.get('bulk_action_serializers')
    default_media_type = config.get('default_media_type')
    search_index = config.get('search_index')
    max_result_window = config.get('max_result_window')

    search_factory = config.get('search_factory_imp', default_search_factory)
    search_factory = obj_or_import_string(search_factory)

    workflow_object_serializers = {
        mime: obj_or_import_string(func)
        for mime, func in workflow_object_serializers.items()
    }
    search_serializers = {
        mime: obj_or_import_string(func)
        for mime, func in search_serializers.items()
    }
    bulk_action_serializers = {
        mime: obj_or_import_string(func)
        for mime, func in bulk_action_serializers.items()
    }
    action_serializers = {
        mime: obj_or_import_string(func)
        for mime, func in action_serializers.items()
    }
    file_serializers = {
        mime: obj_or_import_string(func)
        for mime, func in file_serializers.items()
    }

    list_view = WorkflowsListResource.as_view(
        WorkflowsListResource.view_name,
        search_serializers=search_serializers,
        workflow_object_serializers=workflow_object_serializers,
        default_media_type=default_media_type,
        search_index=search_index,
        search_factory=search_factory,
        max_result_window=max_result_window
    )
    list_route = config.get('list_route')

    item_view = WorkflowObjectResource.as_view(
        WorkflowObjectResource.view_name,
        serializers=workflow_object_serializers,
        default_media_type=default_media_type,
    )
    item_route = config.get('item_route')

    actions_segment = "action/<any(resolve,restart,resume,edit):action>"

    action_route = os.path.join(
        item_route,
        actions_segment
    )
    action_view = WorkflowActionResource.as_view(
        WorkflowActionResource.view_name,
        serializers=action_serializers,
        default_media_type=default_media_type,
    )

    bulk_action_route = os.path.join(
        list_route,
        actions_segment
    )
    bulk_action_view = WorkflowBulkActionResource.as_view(
        WorkflowBulkActionResource.view_name,
        serializers=bulk_action_serializers,
        default_media_type=default_media_type,
    )

    file_item_route = config.get('file_item_route')
    file_item_view = WorkflowFileResource.as_view(
        WorkflowFileResource.view_name,
        serializers=file_serializers,
        default_media_type=default_media_type,
    )

    file_list_route = config.get('file_list_route')
    file_list_view = WorkflowFilesResource.as_view(
        WorkflowFilesResource.view_name,
        serializers=file_serializers,
        default_media_type=default_media_type,
    )

    views = [
        dict(rule=list_route, view_func=list_view),
        dict(rule=item_route, view_func=item_view),
        dict(rule=action_route, view_func=action_view),
        dict(rule=bulk_action_route, view_func=bulk_action_view),
        dict(rule=file_list_route, view_func=file_list_view),
        dict(rule=file_item_route, view_func=file_item_view),
    ]

    for rule in views:
        blueprint.add_url_rule(**rule)

    for proc in context_processors:
        blueprint.context_processor(obj_or_import_string(proc))

    return blueprint


def pass_workflow_object(f):
    """Retrieve workflow object to use in views."""
    @wraps(f)
    def inner(self, object_id, *args, **kwargs):
        try:
            workflow_ui_object = workflow_api_class.get_record(object_id)
        except WorkflowsMissingObject:
            return abort(404)
        return f(
            self,
            workflow_ui_object=workflow_ui_object,
            *args,
            **kwargs
        )
    return inner


class WorkflowsListResource(ContentNegotiatedMethodView):
    """Resource for records listing."""

    view_name = 'workflow_list'

    def __init__(self, read_permission_factory=None,
                 create_permission_factory=None,
                 workflow_object_serializers=None,
                 search_index=None, search_type=None,
                 record_loaders=None,
                 search_serializers=None, default_media_type=None,
                 max_result_window=None, search_factory=None,
                 item_links_factory=None, workflow_api_class=None, **kwargs):
        """Constructor."""
        super(WorkflowsListResource, self).__init__(
            method_serializers={
                'GET': search_serializers,
                'POST': workflow_object_serializers,
            },
            default_method_media_type={
                'GET': default_media_type,
                'POST': default_media_type,
            },
            default_media_type=default_media_type,
            **kwargs)
        self.searcher = RecordsSearch(
            index=search_index,
            doc_type=search_type
        ).params(version=True)
        self.max_result_window = max_result_window
        self.search_factory = partial(search_factory, self)

    @action_read_permission.require(http_exception=403)
    def get(self, **kwargs):
        """Search records.

        :returns: the search result containing hits and aggregations as
        returned by invenio-search.
        """
        page = request.values.get('page', 1, type=int)
        size = request.values.get('size', 10, type=int)
        if page * size >= self.max_result_window:
            raise RESTException("Too many results to show!")

        urlkwargs = dict()
        search = self.searcher[(page-1)*size:page*size]

        search, qs_kwargs = self.search_factory(search)

        urlkwargs.update(qs_kwargs)
        current_app.logger.debug(json.dumps(search.to_dict(), indent=4))
        # Execute search
        search_result = search.execute()

        # Generate links for prev/next
        urlkwargs.update(
            size=size,
            _external=True,
        )
        endpoint = '.{0}'.format(self.view_name)
        links = dict(self=url_for(endpoint, page=page, **urlkwargs))
        if page > 1:
            links['prev'] = url_for(endpoint, page=page-1, **urlkwargs)
        if size * page < int(search_result.hits.total) and \
                size * page < self.max_result_window:
            links['next'] = url_for(endpoint, page=page+1, **urlkwargs)

        return self.make_response(
            search_result=search_result.to_dict(),
            links=links,
        )


class WorkflowObjectResource(ContentNegotiatedMethodView):
    """Resource for workflow items."""

    view_name = 'workflow_item'

    def __init__(self, resolver=None, read_permission_factory=None,
                 update_permission_factory=None,
                 delete_permission_factory=None, default_media_type=None,
                 links_factory=None,
                 loaders=None,
                 **kwargs):
        """Constructor."""
        super(WorkflowObjectResource, self).__init__(
            method_serializers={
                'DELETE': {'*/*': lambda *args: make_response(*args), },
            },
            default_method_media_type={
                'GET': default_media_type,
                'PUT': default_media_type,
                'DELETE': '*/*',
                'PATCH': default_media_type,
            },
            default_media_type=default_media_type,
            **kwargs)

    @pass_workflow_object
    @action_read_permission.require(http_exception=403)
    def get(self, workflow_ui_object, **kwargs):
        """Get a record.

        :param workflow_ui_object: workflow_api_class object.
        :returns: The requested record.
        """
        return self.make_response(workflow_ui_object)

    @pass_workflow_object
    @action_write_permission.require(http_exception=403)
    def put(self, workflow_ui_object, **kwargs):
        """Replace a workflow object.

        :param workflow_ui_object: workflow_api_class object.
        :returns: The modified record.
        """
        data = request.get_json()
        if data:
            workflow_ui_object.clear()
            workflow_ui_object.update(data)
            workflow_ui_object.commit()
            db.session.commit()

        return self.make_response(workflow_ui_object)

    @pass_workflow_object
    @action_write_permission.require(http_exception=403)
    def delete(self, workflow_ui_object, **kwargs):
        """Delete a workflow object.

        :param workflow_ui_object: Workflow object.
        """
        try:
            workflow_ui_object.delete()
            db.session.commit()
        except Exception:
            db.session.rollback()
            current_app.logger.exception('Failed to delete workflow object.')
            abort(500)
        return '', 204


class WorkflowActionResource(ContentNegotiatedMethodView):
    """Workflow actions resource."""

    view_name = 'workflows_action'

    def __init__(self, serializers, *args, **kwargs):
        """Constructor."""
        super(WorkflowActionResource, self).__init__(
            serializers,
            *args,
            **kwargs
        )

    @pass_workflow_object
    @action_write_permission.require(http_exception=403)
    def post(self, workflow_ui_object, action, *args, **kwargs):
        """Post."""
        kwargs['request_data'] = request.json
        kwargs['id_user'] = current_user.get_id()

        response = getattr(workflow_ui_object, action)(*args, **kwargs)
        db.session.commit()

        return self.make_response(dict(
            acknowledged=True,
            result=response,
            action=action,
        ), 200)


class WorkflowFilesResource(ContentNegotiatedMethodView):
    """Workflow file item resource."""

    view_name = 'workflows_file_list'

    def __init__(self, serializers, *args, **kwargs):
        """Constructor."""
        super(WorkflowFilesResource, self).__init__(
            serializers,
            *args,
            **kwargs
        )

    @pass_workflow_object
    @action_read_permission.require(http_exception=403)
    def get(self, workflow_ui_object, *args, **kwargs):
        """Get."""
        return self.make_response(workflow_ui_object.files)


class WorkflowFileResource(ContentNegotiatedMethodView):
    """Workflow actions resource."""

    view_name = 'workflows_file_item'

    def __init__(self, serializers, *args, **kwargs):
        """Constructor."""
        super(WorkflowFileResource, self).__init__(
            serializers,
            *args,
            **kwargs
        )

    @pass_workflow_object
    @action_read_permission.require(http_exception=403)
    def get(self, workflow_ui_object, key, *args, **kwargs):
        """Get."""
        file_obj = workflow_ui_object.files[key]
        return self.make_response(file_obj)


class WorkflowBulkActionResource(ContentNegotiatedMethodView):
    """Workflow bulk-actions resource."""

    view_name = 'bulk_workflows_action'

    def __init__(self, serializers, *args, **kwargs):
        """Constructor."""
        super(WorkflowBulkActionResource, self).__init__(
            serializers,
            *args,
            **kwargs
        )

    @action_write_permission.require(http_exception=403)
    def post(self, action, *args, **kwargs):
        """Handle deposit action."""
        from flask_login import current_user

        # Add data to kwargs (needed for potential async tasks)
        kwargs['request_data'] = request.json
        kwargs['id_user'] = current_user.get_id()

        object_ids = request.json.get('object_ids', [])

        async_task = resolve_actions.delay(object_ids, action, *args, **kwargs)

        return self.make_response(dict(
            acknowledged=True,
            task_id=async_task.id,
        ))



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""Search functions for workflows UI interface."""

from __future__ import absolute_import, print_function

import copy

from elasticsearch_dsl import Q

from flask import current_app, request

from werkzeug.datastructures import MultiDict


def terms_filter(field):
    """Create a term filter."""
    def inner(values):
        return Q('terms', **{field: values})
    return inner


def _create_filter_dsl(urlkwargs, definitions):
    """Create a filter DSL expression."""
    filters = []
    for name, filter_factory in definitions.items():
        values = request.values.getlist(name, type=str)
        if values:
            filters.append(filter_factory(values))
            for v in values:
                urlkwargs.add(name, v)

    return (filters, urlkwargs)


def _post_filter(search, urlkwargs, definitions):
    """Ingest post filter in query."""
    filters, urlkwargs = _create_filter_dsl(urlkwargs, definitions)

    for filter_ in filters:
        search = search.post_filter(filter_)

    return (search, urlkwargs)


def _query_filter(search, urlkwargs, definitions):
    """Ingest query filter in query."""
    filters, urlkwargs = _create_filter_dsl(urlkwargs, definitions)

    for filter_ in filters:
        search = search.filter(filter_)

    return (search, urlkwargs)


def _aggregations(search, definitions):
    """Add aggregations to query."""
    if definitions:
        for name, agg in definitions.items():
            search.aggs[name] = agg
    return search


def parse_sort_field(field_value):
    """Parse a URL field.

    :param field_value: Field value (e.g. 'key' or '-key').
    :returns: Tuple of (field, ascending).
    """
    if field_value.startswith("-"):
        return (field_value[1:], False)
    return (field_value, True)


def reverse_order(order_value):
    """Reserve ordering of order value (asc or desc).

    :param order_value: Either the string ``asc`` or ``desc``.
    :returns: Reverse sort order of order value.
    """
    if order_value == 'desc':
        return 'asc'
    elif order_value == 'asc':
        return 'desc'
    return None


def eval_field(field, asc):
    """Evaluate a field for sorting purpose.

    :param field: Field definition (string, dict or callable).
    :param asc: ``True`` if order is ascending, ``False`` if descending.
    """
    if isinstance(field, dict):
        if asc:
            return field
        else:
            # Field should only have one key and must have an order subkey.
            field = copy.deepcopy(field)
            key = list(field.keys())[0]
            field[key]['order'] = reverse_order(field[key]['order'])
            return field
    elif callable(field):
        return field(asc)
    else:
        key, key_asc = parse_sort_field(field)
        if not asc:
            key_asc = not key_asc
        return {key: {'order': 'asc' if key_asc else 'desc'}}


def default_facets_factory(search, index):
    """Add facets to query."""
    urlkwargs = MultiDict()

    facets = current_app.config['WORKFLOWS_UI_REST_FACETS'].get(index)

    if facets is not None:
        # Aggregations.
        search = _aggregations(search, facets.get("aggs", {}))

        # Query filter
        search, urlkwargs = _query_filter(
            search, urlkwargs, facets.get("filters", {}))

        # Post filter
        search, urlkwargs = _post_filter(
            search, urlkwargs, facets.get("post_filters", {}))

    return (search, urlkwargs)


def default_search_factory(self, search, **kwargs):
    """Create default ES query based on query-string pattern."""
    if 'q' in kwargs:
        query_string = kwargs['q']
    else:
        query_string = request.values.get('q', '')

    if not query_string:
        # Assume empty query == match all
        search = search.query(Q('match_all'))
    else:
        search = search.query(Q('query_string',
                                query=query_string,
                                allow_leading_wildcard=False))

    search_index = search._index[0]

    search, urlkwargs = default_facets_factory(search, search_index)
    search, sortkwargs = default_sorter_factory(search, search_index)
    for key, value in sortkwargs.items():
        urlkwargs.add(key, value)

    urlkwargs.add('q', query_string)
    return (search, urlkwargs)


def default_sorter_factory(search, index):
    """Sort a query.

    :param query: Search query.
    :param index: Index to search in.
    :returns: Tuple of (query, URL arguments).
    """
    sort_arg_name = 'sort'
    urlfield = request.values.get(sort_arg_name, '', type=str)

    # Get default sorting if sort is not specified.
    if not urlfield:
        has_query = request.values.get('q', type=str)
        urlfield = current_app.config['WORKFLOWS_UI_REST_DEFAULT_SORT'].get(
            index, {}).get('query' if has_query else 'noquery', '')

    # Parse sort argument
    key, asc = parse_sort_field(urlfield)

    # Get sort options
    sort_options = current_app.config['WORKFLOWS_UI_REST_SORT_OPTIONS'].get(
        index, {}).get(key)
    if sort_options is None:
        return (search, {})

    # Get fields to sort query by
    search = search.sort(
        *[eval_field(f, asc) for f in sort_options['fields']]
    )
    return (search, {sort_arg_name: urlfield})



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2017 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""CLI for invenio-workflows-ui."""

from __future__ import absolute_import, print_function

import pprint

import click
import elasticsearch
from flask import current_app
from flask.cli import with_appcontext
from invenio_search import current_search_client as es
from invenio_workflows.api import WorkflowObject
from invenio_workflows.models import WorkflowObjectModel
from itertools import chain
from time import sleep

from .proxies import workflow_api_class
from .tasks import batch_reindex


@click.group()
def holdingpen():
    """Manage holdingpen."""


def next_batch(iterator, batch_size):
    """Get first batch_size elements from the iterable, or remaining if less.

    :param iterator: the iterator for the iterable
    :param batch_size: size of the requested batch
    :return: batch (list)
    """
    batch = []

    try:
        for idx in range(batch_size):
            batch.append(next(iterator))
    except StopIteration:
        pass

    return batch


@holdingpen.command()
@click.option('--yes-i-know', is_flag=True)
@click.option('-t', '--data-type', multiple=True, required=True)
@click.option('-s', '--batch-size', default=200)
@click.option('-q', '--queue-name', default='indexer_task')
@with_appcontext
def reindex(yes_i_know, data_type, batch_size, queue_name):
    """Reindex all records in a parallel manner.

    :param yes_i_know: if True, skip confirmation screen
    :param data_type: workflow data type.
    :param batch_size: number of documents per batch sent to workers.
    :param queue_name: name of the celery queue
    """
    if not yes_i_know:
        click.confirm(
            'Do you really want to reindex the workflows?',
            abort=True,
        )

    click.secho('Sending workflows to the indexing queue...', fg='green')

    query = WorkflowObjectModel.query.filter(
        WorkflowObjectModel.data_type.in_(data_type)
    )
    request_timeout = current_app.config.get('INDEXER_BULK_REQUEST_TIMEOUT')
    all_tasks = []

    with click.progressbar(
        query.yield_per(1000),
        length=query.count(),
        label='Scheduling indexing tasks'
    ) as items:
        batch = next_batch(items, batch_size)

        while batch:
            task = batch_reindex.apply_async(
                kwargs={
                    'workflow_ids': [item.id for item in batch],
                    'request_timeout': request_timeout,
                },
                queue=queue_name,
            )

            all_tasks.append(task)
            batch = next_batch(items, batch_size)

    click.secho('Created {} tasks.'.format(len(all_tasks)), fg='green')

    with click.progressbar(
        length=len(all_tasks),
        label='Indexing workflows'
    ) as progressbar:
        def _finished_tasks_count():
            return len(filter(lambda task: task.ready(), all_tasks))

        while len(all_tasks) != _finished_tasks_count():
            sleep(0.5)
            progressbar.pos = _finished_tasks_count()
            progressbar.update(0)

    successes = sum([task.result.get('success', 0) for task in all_tasks])
    failures = sum([task.result.get('failures', []) for task in all_tasks], [])

    color = 'red' if failures else 'green'
    click.secho(
        'Reindexing failed for {} records, succeeded for {}.'.format(
            len(failures),
            successes
        ),
        fg=color,
    )
    if failures:
        click.secho('Errors: {}'.format(failures), fg='red')



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Invenio module which acts as a UI layer for invenio-workflows."""

from setuptools import find_packages, setup


TESTS_REQUIRE = [
    'check-manifest>=0.25',
    'coverage>=4.0',
    'isort>=4.2.2',
    'pydocstyle>=1.0.0',
    'pytest-cache>=1.0',
    'pytest-cov>=1.8.0',
    'pytest-pep8>=1.0.6',
    'pytest>=2.8.0',
]

EXTRAS_REQUIRE = {
    'docs': [
        'Sphinx>=1.3',
    ],
    'ui': [
        'invenio-search-ui>=1.0.0a4',
    ],
    'postgresql': [
        'invenio-db[postgresql]>=1.0.0a9',
    ],
    'mysql': [
        'invenio-db[mysql]>=1.0.0a9',
    ],
    'sqlite': [
        'invenio-db>=1.0.0a9',
    ],
    'elasticsearch2': [
        'elasticsearch-dsl~=2.0',
    ],
    'elasticsearch5': [
        'elasticsearch-dsl~=5.0',
    ],
    'tests': TESTS_REQUIRE,
}

EXTRAS_REQUIRE['all'] = []
for name, reqs in EXTRAS_REQUIRE.items():
    if name in ('postgresql', 'sqlite', 'elasticsearch2', 'elasticsearch5'):
        continue
    EXTRAS_REQUIRE['all'].extend(reqs)

SETUP_REQUIRES = [
    'autosemver~=0.2,>=0.2',
    'Babel>=1.3',
    'pytest-runner>=2.6.2',
]


INSTALL_REQUIRES = [
    'autosemver~=0.2,>=0.2',
    'Flask-BabelEx>=0.9.2',
    'invenio-access~=1.0,>=1.0.1',
    'invenio-search>=1.0.0a5',
    'invenio-indexer>=1.0.0a5',
    'invenio-rest',
    'flask-login',
    'invenio-workflows~=7.0',
    'flask-principal',
]

URL = 'https://github.com/inveniosoftware-contrib/invenio-workflows-ui'


if __name__ == '__main__':
    setup(
        name='invenio-workflows-ui',
        description=__doc__,
        long_description=open('README.rst').read(),
        keywords='invenio workflows gui interface content curation',
        license='GPLv2',
        author='CERN',
        author_email='info@invenio-software.org',
        url=URL,
        autosemver={
            'bugtracker_url': URL + '/issues/',
        },
        packages=find_packages(),
        zip_safe=False,
        include_package_data=True,
        platforms='any',
        entry_points={
            'invenio_base.apps': [
                (
                    'invenio_workflows_ui '
                    '= invenio_workflows_ui:InvenioWorkflowsUI'
                ),
            ],
            'invenio_base.api_apps': [
                (
                    'invenio_workflows_ui '
                    '= invenio_workflows_ui:InvenioWorkflowsUIREST'
                ),
            ],
            'invenio_access.actions': [
                (
                    'holdingpen_read_access'
                    ' = invenio_workflows_ui.permissions:action_read_access'
                ),
                (
                    'holdingpen_write_access'
                    ' = invenio_workflows_ui.permissions:action_write_access'
                ),
                (
                    'holdingpen_admin_access'
                    ' = invenio_workflows_ui.permissions:action_admin_access'
                ),
            ],
            'invenio_celery.tasks': [
                'invenio_workflows_ui = invenio_workflows_ui.tasks',
            ]
        },
        extras_require=EXTRAS_REQUIRE,
        install_requires=INSTALL_REQUIRES,
        setup_requires=SETUP_REQUIRES,
        tests_require=TESTS_REQUIRE,
        classifiers=[
            'Environment :: Web Environment',
            'Intended Audience :: Developers',
            'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',
            'Operating System :: OS Independent',
            'Programming Language :: Python',
            'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
            'Topic :: Software Development :: Libraries :: Python Modules',
            'Programming Language :: Python :: 2',
            'Programming Language :: Python :: 2.7',
            'Programming Language :: Python :: 3',
            'Programming Language :: Python :: 3.5',
            'Development Status :: 1 - Planning',
        ],
    )
